/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
   *	bookmarkTools class structure:
   *
   *	root: BookmarkTools
   *
   *	objects:
   *	BookmarkTools -> utils -> Browser, Fileio
   *
   *	'loose' functions:
   *	BookmarkTools -> namespace, include
   *	BookmarkTools -> overlay
   *	BookmarkTools -> panel -> index
   *	BookmarkTools -> panel -> errors
   *	BookmarkTools -> panel -> duplicates
   *
   */
 
/** components shortcuts */
var Cc = Components.classes;
var Ci = Components.interfaces;
var Cr = Components.results;
var Cu = Components.utils;
 
/** supported browser types */
const FIREFOX_ID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
const SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";
 
/** The global BookmarkTools namespace. */
if(typeof BookmarkTools == "undefined" || !BookmarkTools || BookmarkTools == null){
	var BookmarkTools = {};
}

/** Returns the specified namespace and creates it if it doesn't exists */
BookmarkTools.namespace = function(param){
	var ns = null, subspaces = [];

	//force arguments as strings
	subspaces = ("" + param).split('.');
	ns = BookmarkTools;

	//append cascade of objects
	for(var i = 0, l = subspaces.length ;i < l; i++){
		var entry = ns[subspaces[i]];
		if(typeof entry == "undefined" || !entry || entry == null){
			ns[subspaces[i]] = {};
		}
		ns = ns[subspaces[i]];
	}
	
	return ns;
}

/** Includes *.js files */
BookmarkTools.include = function(name){
	var HEADER = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
	try {
		HEADER.loadSubScript("chrome://bookmarktools/content/" + name);
	} catch(e){
		alert("ERROR: Can't import *.js file (BookmarkTools.include)!");
	}
}

/** Report handler that shows detailed exception */
BookmarkTools.reportError = function(error, msg){
	try {
		alert(
			(msg ? (msg + "\n\n") : "") + 
			"message: " + error.message + "\n" +
			"fileName: " + error.fileName + "\n" + 
			"lineNumber: " + error.lineNumber + "\n" + 
			"stack: " + error.stack
		);
	} catch(e){
		alert("ERROR: Can't execute exception report (BookmarkTools.reportError)!");
	}
}

/** define general namespaces */
BookmarkTools.namespace("globals");
BookmarkTools.namespace("utils.Browser");

BookmarkTools.globals.lockKey = -1;

BookmarkTools.utils.Browser.FIREFOX_ID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
BookmarkTools.utils.Browser.SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";
BookmarkTools.utils.Browser.browserID = "";

BookmarkTools.utils.Browser.init = function(){
	var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
	this.browserID = appInfo.ID;
};

BookmarkTools.utils.Browser.isSeaMonkey = function(){
	return this.browserID == this.SEAMONKEY_ID;
};
	
BookmarkTools.utils.Browser.isFirefox = function(){
	return this.browserID == this.FIREFOX_ID;
};

BookmarkTools.utils.Browser.init();
