/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("panel.index");
 
const truncateNum = 60;

BookmarkTools.panel.index = {
	
	init : function(){ 
	
		/** general settings, compatible with all build modes */
		BookmarkTools.panel.errors.loadErrorPrefs(); 
		BookmarkTools.panel.duplicates.loadDuplicatePrefs(); 
		this.startLookup(); 
		
		// widgets
		var buildMode = $("bookmarkTools.pane.radiogroupBT").selectedItem.id;
		var intervalSlider = $("bookmarkTools.pane.intervalSlider");
		var buildButton = $("bookmarkTools.pane.buildButton");
		var deleteButton = $("bookmarkTools.pane.deleteButton");
	
		var checkButton = $("bookmarkTools.pane.checkButton");
		var discardErrorsButton = $("bookmarkTools.pane.discardErrorButton");
		var discardDuplicatesButton = $("bookmarkTools.pane.discardDuplicateButton");

		/** automatic build mode */
		if(buildMode == "bookmarkTools.pane.automatic"){
			/** only works when indexer is running */
			if(Prefs.getBool('isRunning')){
				intervalSlider.disabled = true;
				$("bookmarkTools_pane_loadFolder").disabled = true;
				$("bookmarkTools.pane.loadFolderCheck").disabled = true;
				$("bookmarkTools.pane.radiogroupBT").disabled = true;
			}else { /** else */
				intervalSlider.disabled = false; 
			}
			// always
			buildButton.disabled = true; 
			deleteButton.disabled = true; 
			checkButton.disabled = true;
			discardErrorsButton.disabled = true;
			discardDuplicatesButton.disabled = true;
		}
		
		/** manual build mode */
		if(buildMode == "bookmarkTools.pane.manual"){
			intervalSlider.disabled = true; 
			buildButton.disabled = false;
			deleteButton.disabled = false;
			checkButton.disabled = false;
			discardErrorsButton.disabled = false;
			discardDuplicatesButton.disabled = false;
		}
		
		// load preferences
		$("bookmarkTools.pane.intervalSlider").value = Prefs.getInt("indexDelay");
		var path = Prefs.getFile("indexPath").path;
		$("bookmarkTools.pane.pathTooltip").firstChild.value = path;
		$("bookmarkTools.pane.pathInfo").value = StringUtils.truncate(path, truncateNum, ' [..]');
		$("bookmarkTools_pane_loadFolder").disabled = Prefs.getBool("indexPathDefault");
	},
	
	uninit : function(){
		// set slider preference
		Prefs.setInt("indexDelay", $("bookmarkTools.pane.intervalSlider").value);
	},
	
	showRestartMessage : function(item){
		var currentBuildType = Prefs.getInt('buildType');
		if(item.value != currentBuildType){
			$('bookmarkTools.restartMsg').setAttribute("style", "color: red; visibility: visible;");
		}else {
			$('bookmarkTools.restartMsg').setAttribute("style", "visibility: hidden;");
		}
	},
	
	/*
		Sets the current choosed build type and resets
		preferences if the autimatic mode was selected.

		@param item radiogroup to get the selected item
	*/
	setBuildTypePrefs : function(item){
		if(item.selectedItem.id == 'bookmarkTools.pane.automatic')
			Prefs.setBool('firstRun', true);
	},
	
	setLocationValue : function(slider){
		var elem = $("bookmarkTools.pane.info");
		var val = elem.value.split(" ")[1];
		elem.value = slider.value + " " + val;
	},
	
	setUserDefinedPath : function(checkbox){
		if(!checkbox.checked){
			$("bookmarkTools_pane_loadFolder").disabled = true;
			Prefs.remove("alreadyAddedBookmarks");
		}else {
			$("bookmarkTools_pane_loadFolder").disabled = false;
		}
	},
	
	setIndexPath : function(){
		var nsIFilePicker = Ci.nsIFilePicker;
		var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		fp.init(window, "Select folder", nsIFilePicker.modeGetFolder);
		var res = fp.show();
		if (res == nsIFilePicker.returnOK){
			var path = fp.file.path;
			Prefs.setFile("indexPath", fp.file);
			$("bookmarkTools.pane.pathTooltip").firstChild.value = path;
			$("bookmarkTools.pane.pathInfo").value = StringUtils.truncate(path, truncateNum, ' [..]');
			Prefs.remove("alreadyAddedBookmarks");
		}
	},
	
	/*
		Shows currently indexed bookmarks.
		
		Note:
			Never change indexed bookmarks while lookup, 
			because the delete function recieves the tree indexes that are
			used to refer to the corresponding index document.
	*/
	startLookup : function(){
		try {
			var array = Index._lookup();
			var e = array.enumerate();

			var errors = Prefs.getBranch('invBm').toHash();

			var tree_indexed = $("bookmarkTools.pane.indexTree");
			tree_indexed.clear();
			
			var untraceableBookmarks = new ExtendedHash();
			var pos = 0;
			
			while (e.hasMoreElements()) {
				var lookupNode = e.getNext().QueryInterface(Ci.ILookupNode);

				var status = STATUS_SUCCESS; 
				if(!Book.isBookmarked(lookupNode.primaryKey)){
					status = STATUS_UNTRACEABLE;
					untraceableBookmarks.put(lookupNode.primaryKey, pos);
					
					if(Prefs.getBool('cleanupUntraceableBookmarks')){
						pos++;
						continue;
					}
				}else if(errors.contains(lookupNode.primaryKey)){
					status = STATUS_CORRUPTED;
				}
				
				tree_indexed.add(lookupNode.name, lookupNode.primaryKey, status);
				pos++;
			}
			
			//checks preferences if cleanup flag is set
			//and removes all untraceable bookmarks
			this.doCleanup(untraceableBookmarks);
			
			$("bookmarkTools.pane.indexCtr").value = "(" + tree_indexed.size + ")";
		} catch(e){
			BookmarkTools.reportError(e);
		}
	},
	
	doCleanup : function(untraceableBookmarks){
		if(Prefs.getBool('cleanupUntraceableBookmarks') && !untraceableBookmarks.isEmpty()){
			Index._delete(untraceableBookmarks);
		}
	},
	
	doCheck : function(){
		//clear prefs
		Prefs.getBranch('invBm').clear();
		Prefs.getBranch('dupBm').clear();
		
		//clear last log files
		BookmarkTools.utils.Fileio.clearLogs();
		
		//clear tree and button information
		$("bookmarkTools.pane.errorTree").clear();
		$("bookmarkTools.pane.errorCtr").value = "(0)";
		
		$("bookmarkTools.pane.otherTree").clear();
		$("bookmarkTools.pane.otherCtr").value = "(0)";
		
		$("bookmarkTools.pane.duplicateTree").clear();
		$("bookmarkTools.pane.duplicateCtr").value = "(0)";
		
		//init crawler
		Crawler.init();
		Crawler.setBookmarks(Book.getResourceHash());

		//init filter
		Filter.init();
		Filter.setCrawler(Crawler);
		Filter.setCheck(true);
		Filter.setView(View);
		
		Index.setMode(MANUAL);
		
		var retVals = {filter : Filter};
		openDialog("chrome://bookmarktools/content/bookmarkToolsProgress.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", retVals);
	},
	
	doAdd : function(){
		//clear prefs
		Prefs.getBranch('invBm').clear();
		Prefs.getBranch('dupBm').clear();
		
		//clear last log files
		BookmarkTools.utils.Fileio.clearLogs();
		
		//clear tree and button information
		$("bookmarkTools.pane.errorTree").clear();
		$("bookmarkTools.pane.errorCtr").value = "(0)";
		
		$("bookmarkTools.pane.indexTree").clear();
		$("bookmarkTools.pane.indexCtr").value = "(0)";
		
		$("bookmarkTools.pane.otherTree").clear();
		$("bookmarkTools.pane.otherCtr").value = "(0)";
		
		$("bookmarkTools.pane.duplicateTree").clear();
		$("bookmarkTools.pane.duplicateCtr").value = "(0)";
		
		//clear search dir
		BookmarkTools.utils.Fileio.clear(BookmarkTools.utils.Fileio.getSearchDir());
		
		var bookmarks = Book.getResourceHash();

		//init crawler
		Crawler.init();
		Crawler.setBookmarks(bookmarks);

		//init filter
		Filter.init();
		
		//bind with index
		Index.setMode(MANUAL);
		Index.setFilter(Filter);
		
		Filter.setCrawler(Crawler);
		Filter.setView(View);
		
		var retVals = {index : Index};
		openDialog("chrome://bookmarktools/content/bookmarkToolsProgress.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", retVals);
	},
	
	doInfo : function(){
		var args = {
			bookmarks: Book.getSize(),
			diskSize: BookmarkTools.utils.Fileio.formatBytes(BookmarkTools.utils.Fileio.getDirectoryInfo(BookmarkTools.utils.Fileio.getSearchDir())),
			numberOfDocuments: Index._size(),
			buildDate: parseInt(Prefs.getChar('intIndexDate')),
			delay: Index.delay,
			mode: Index.mode
		};
		openDialog("chrome://bookmarktools/content/bookmarkToolsInfo.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", args);
	},
	
	tryBuildAutomatic : function(){
		//checks if the build type 'AUTOMATIC' is set and 
		//if the current time is greater then the last build time + delay
		if(Index.isValidBuildTime()){
			
			if(BookmarkTools.globals.lockKey != -1){
			
				if(Prefs.getBool('firstRun')){ //first run?
					//clear prefs
					Prefs.getBranch('invBm').clear();
					Prefs.getBranch('dupBm').clear();
					
					//clear last log files
					BookmarkTools.utils.Fileio.clearLogs();
					
					//clear build directory
					BookmarkTools.utils.Fileio.clear(BookmarkTools.utils.Fileio.getBuildDir());
					
					//generate log file
					var file = BookmarkTools.utils.Fileio.generateFile(BookmarkTools.utils.Fileio.getBuildDir(), "alreadyAddedBookmarks.log");
					Prefs.setFile("alreadyAddedBookmarks", file);
				}
			
				//init crawler
				Crawler.init();

				//init filter
				Filter.init();
				var indexState = Filter.getBookmarksFromFile();
				Crawler.setBookmarks(indexState.remainingBookmarks);

				//bind with index
				Index.setMode(AUTOMATIC);
				Index.setFilter(Filter);
				
				Filter.setCrawler(Crawler, indexState.currentIndexed, indexState.allBookmarks);
		
				BrowserView.init();
				Filter.setView(BrowserView);
				
				Index._build();
			
				//sets first run to false
				Prefs.setBool('firstRun', false);
				Prefs.setBool('isRunning', true);
			}
		}
	},
	
	doDelete : function(){
		var tree = $('bookmarkTools.pane.indexTree');
		var indexes = tree.selection;
		tree.unselect;

		var nodes = new ExtendedHash();
		for(var i = 0, l = indexes.length ;i < l; i++){
			var index = indexes[i];
			var url = tree.getItemURL(index);
			nodes.put(url, index);
		}
		
		//set path to search directory
		Index.setMode(MANUAL);
		Index._delete(nodes);

		this.startLookup();
	}
	
};
 